"""Spanish dialect"""

pyes = {
    "False": "Falso",
    "None": "Nada",
    "True": "Verdadero",
    "and": "y",
    "as": "como",
    "assert": "afirmar",
    "async": "async",  # do not translate
    "await": "await",  # as these are not for beginners
    "break": "interrumpir",
    "class": "clase",
    "continue": "continuar",
    "def": "def",
    "del": "eliminar",
    "elif": "osi",
    "else": ["sino", "nointerrumpir"],
    "except": "excepto",
    "finally": "finalmente",
    "for": "para",
    "from": "de",
    "global": "global",
    "if": "si",
    "import": "importar",
    "in": "en",
    "is": "es",
    "lambda": "función",
    "nonlocal": "nolocal",
    "not": "no",
    "or": "o",
    "pass": "seguir",
    "raise": "elevar",
    "return": "retornar",
    "try": "intentar",
    "while": "mientras",
    "with": "con",
    "yield": "ceder",
    # Special loop keywords
    "repeat": "repetir",
    "forever": "parasiempre",
    "until": "hasta",
    # a few builtins useful for beginners
    "input": "entrada",
    "print": "imprimir",
    "range": "intervalo",
    "exit": "salir",  # useful for console - translation should be checked
    # a well-known python expression
    '__name__ == "__main__"': "noimportado",
}
